@@ -166,6 +166,8 @@ additional = {
 required_preprocessing = [
     "missing_values_inputation",
     "convert_categorical",
+    "datetime_transform",
+    "text_transform",
     "target_as_integer",
 ]
 
@@ -208,6 +210,8 @@ AlgorithmsRegistry.add(
 regression_required_preprocessing = [
     "missing_values_inputation",
     "convert_categorical",
+    "datetime_transform",
+    "text_transform",
     "target_scale",
 ]
 
