@@ -11,6 +11,9 @@ from supervised.preprocessing.label_encoder import LabelEncoder
 from supervised.preprocessing.label_binarizer import LabelBinarizer
 from supervised.preprocessing.datetime_transformer import DateTimeTransformer
 from supervised.preprocessing.text_transformer import TextTransformer
+from supervised.preprocessing.goldenfeatures_transformer import (
+    GoldenFeaturesTransformer,
+)
 from supervised.preprocessing.exclude_missing_target import ExcludeRowsMissingTarget
 from supervised.algorithms.registry import (
     BINARY_CLASSIFICATION,
@@ -46,6 +49,9 @@ class Preprocessing(object):
         self._remove_columns = []
         self._datetime_transforms = []
         self._text_transforms = []
+        self._golden_features = None
+        self._add_random_feature = self._params.get("add_random_feature", False)
+        self._drop_features = self._params.get("drop_features", [])
 
     def _exclude_missing_targets(self, X=None, y=None):
         # check if there are missing values in target column
@@ -125,6 +131,11 @@ class Preprocessing(object):
             X_train.drop(cols_to_remove, axis=1, inplace=True)
         self._remove_columns = cols_to_remove
 
+        numeric_cols = [] # get numeric cols before text transformations
+        # needed for golden features
+        if X_train is not None and "golden_features" in self._params:
+            numeric_cols = X_train.select_dtypes(include="number").columns.tolist()
+
         # there can be missing values in the text data,
         # but we don't want to handle it by fill missing methods
         # zeros will be imputed by text_transform method
@@ -156,6 +167,17 @@ class Preprocessing(object):
             X_train = missing.transform(X_train)
             self._missing_values += [missing]
 
+        # golden features
+        golden_columns = []
+        if "golden_features" in self._params:
+            results_path = self._params["golden_features"]["results_path"]
+            ml_task = self._params["golden_features"]["ml_task"]
+            # if ml_task in [BINARY_CLASSIFICATION]:
+            self._golden_features = GoldenFeaturesTransformer(results_path, ml_task)
+            self._golden_features.fit(X_train[numeric_cols], y_train)
+            X_train = self._golden_features.transform(X_train)
+            golden_columns = self._golden_features._new_columns
+
         for convert_method in [
             PreprocessingCategorical.CONVERT_INTEGER,
             PreprocessingCategorical.CONVERT_ONE_HOT,
@@ -209,12 +231,30 @@ class Preprocessing(object):
             ):
                 cols_to_process += new_text_columns
 
+            if (
+                len(cols_to_process)
+                and len(golden_columns)
+                and scale_method == Scale.SCALE_NORMAL
+            ):
+                cols_to_process += golden_columns
+
             if len(cols_to_process):
                 scale = Scale(cols_to_process)
                 scale.fit(X_train)
                 X_train = scale.transform(X_train)
                 self._scale += [scale]
 
+        if self._add_random_feature:
+            # -1, 1, with 0 mean
+            X_train["random_feature"] = np.random.rand(X_train.shape[0]) * 2.0 - 1.0
+
+        if self._drop_features:
+            available_cols = X_train.columns.tolist()
+            drop_cols = [c for c in self._drop_features if c in available_cols]
+            if drop_cols:
+                X_train.drop(drop_cols, axis=1, inplace=True)
+            self._drop_features = drop_cols
+
         return X_train, y_train
 
     def transform(self, X_validation, y_validation):
@@ -299,6 +339,11 @@ class Preprocessing(object):
             )
             missing.fit(X_validation)
             X_validation = missing.transform(X_validation)
+
+        # golden features
+        if self._golden_features is not None:
+            X_validation = self._golden_features.transform(X_validation)
+
         for convert in self._categorical:
             if X_validation is not None and convert is not None:
                 X_validation = convert.transform(X_validation)
@@ -311,6 +356,15 @@ class Preprocessing(object):
             if X_validation is not None and scale is not None:
                 X_validation = scale.transform(X_validation)
 
+        if self._add_random_feature:
+            # -1, 1, with 0 mean
+            X_validation["random_feature"] = (
+                np.random.rand(X_validation.shape[0]) * 2.0 - 1.0
+            )
+
+        if self._drop_features and X_validation is not None:
+            X_validation.drop(self._drop_features, axis=1, inplace=True)
+
         return X_validation, y_validation
 
     def inverse_scale_target(self, y):
@@ -450,6 +504,9 @@ class Preprocessing(object):
             if tts:
                 preprocessing_params["text_transforms"] = tts
 
+        if self._golden_features is not None:
+            preprocessing_params["golden_features"] = self._golden_features.to_json()
+
         if self._scale is not None and len(self._scale):
             scs = [sc.to_json() for sc in self._scale if sc.to_json()]
             if scs:
@@ -463,6 +520,13 @@ class Preprocessing(object):
 
         if "ml_task" in self._params:
             preprocessing_params["ml_task"] = self._params["ml_task"]
+
+        if self._add_random_feature:
+            preprocessing_params["add_random_feature"] = True
+
+        if self._drop_features:
+            preprocessing_params["drop_features"] = self._drop_features
+
         return preprocessing_params
 
     def from_json(self, data_json):
@@ -496,6 +560,10 @@ class Preprocessing(object):
                 tt.from_json(tt_params)
                 self._text_transforms += [tt]
 
+        if "golden_features" in data_json:
+            self._golden_features = GoldenFeaturesTransformer()
+            self._golden_features.from_json(data_json["golden_features"])
+
         if "scale" in data_json:
             self._scale = []
             for scale_data in data_json["scale"]:
@@ -514,3 +582,6 @@ class Preprocessing(object):
             self._scale_y.from_json(data_json["scale_y"])
         if "ml_task" in data_json:
             self._params["ml_task"] = data_json["ml_task"]
+
+        self._add_random_feature = data_json.get("add_random_feature", False)
+        self._drop_features = data_json.get("drop_features", [])
